<?php

namespace FiloBlu\Maintenance\Helper;

class Data extends \Magento\Framework\App\Helper\AbstractHelper {

    const COOKIE_NAME = 'filoblu_maintenance_admin_loggedin';

    /**
     * @var \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress
     */
    protected $remoteAddress;
    protected $cookieManager;
    protected $securityConfig;
    protected $adminSessionInfoCollectionFactory;
    protected $encryptor;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    public function __construct(
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\Stdlib\CookieManagerInterface $cookieManager,
        \Magento\Security\Model\ConfigInterface $securityConfig,
        \Magento\Security\Model\ResourceModel\AdminSessionInfo\CollectionFactory $adminSessionInfoCollectionFactory,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor
    ) {

        $this->remoteAddress = $context->getRemoteAddress();
        $this->customerSession = $customerSession;
        $this->cookieManager = $cookieManager;
        $this->securityConfig = $securityConfig;
        $this->adminSessionInfoCollectionFactory = $adminSessionInfoCollectionFactory;
        $this->encryptor = $encryptor;
        parent::__construct($context);
    }

    public function getConfig($config_path, $section = 'filoblu_maintenance_section') {
        return $this->scopeConfig->getValue(
                        $section . '/' . $config_path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    public function isAllowedIp() {
        $isAllowed = false;
 
        if ($this->getConfig("general/allowed_ips") && in_array($this->remoteAddress->getRemoteAddress(), explode(",",trim($this->getConfig("general/allowed_ips"))))) {
            $isAllowed = true;
        }

        return $isAllowed;
    }

    public function isAdminOnlyAllowed() {
        return $this->scopeConfig->isSetFlag('filoblu_maintenance_section/general/admin_only', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    /**
     * Check if cookie contains a valid (hashed) session_id of an admin
     * 
     * @return bool
     */
    public function isValidAdmin() {
        $isValidAdmin = FALSE;

        $hashedAdminSessionInCookie = $this->cookieManager->getCookie(self::COOKIE_NAME, 0);
        if ($hashedAdminSessionInCookie == '0') {
            return $isValidAdmin;
        }

        //load valid admin sessions
        $loggedAdmins = $this->adminSessionInfoCollectionFactory->create();
        $loggedAdmins->filterExpiredSessions($this->securityConfig->getAdminSessionLifetime());
        $loggedAdmins->addFieldToFilter('status', \Magento\Security\Model\AdminSessionInfo::LOGGED_IN);

        foreach($loggedAdmins as $admin) {
            if ($hashedAdminSessionInCookie == $this->encryptor->hash($admin->getSessionId())) {
                $isValidAdmin = TRUE;
                break;
            }
        }
        
        return $isValidAdmin;
    }


    public function allowedUrls(){
        $allowed_urls = '';
        
        if ($urls=trim($this->getConfig("general/allowed_urls"))) {
            
            $allowed_urls = explode("\n" , $urls);
            $allowed_urls = array_filter(array_map('trim', $allowed_urls));
        }

        return $allowed_urls;
    }

}
